@@ -48,6 +48,9 @@ class TopdeskConnector:
 					return response
 			else:
 				return ""
+		else:
+			print("Status code:", response.status_code)
+			print("content:", response.content)
 
 	def valid_connection(self):
 		"""
@@ -213,15 +216,27 @@ class TopdeskConnector:
 		:return:
 		"""
 		# naam: Comparex Nederland B.V.
-		if completed is None:
-			params = f'query=supplier.id=={supplier_id}'
+		if completed is None or completed is False:
+			params = f"query=supplier.id=={supplier_id};processingStatus.name=in=('In behandeling','Toegewezen')"
 		if completed:
-			params = f'query=supplier.id=={supplier_id};completed==True'
-		if completed is False:
-			params = f'query=supplier.id=={supplier_id};completed==False'
+			params = f'query=supplier.id=={supplier_id};processingStatus.name==Gereed'
 
 		return self.make_request('GET', "incidents", params=params)
 
+	def incidents_by_supplier_with_status(self, supplier_id: str, status_name: str):
+		"""
+		Verkrijg alle incidenten toebehorende aan een supplier, met een gegeven statusnaam
+
+		Args:
+			supplier_id: Supplier Id string
+			status_name: status naam van incidenten
+
+		Returns: incidents dict
+
+		"""
+		params = f"query=supplier.id=={supplier_id};processingStatus.name=='{status_name}'"
+		return self.make_request('GET', "incidents", params=params)
+
 	def newest_incidents(self, amount=5, start_offset=0, processing_status_id: str = None):
 		"""
 		Verkrijg een lijst van incidenten, gesorteerd op nieuw naar oud
@@ -300,11 +315,10 @@ class TopdeskConnector:
 
 		return files_dict
 
-	def upload_attachment_to_incident(self, incident_id: int, file_handle):
+	def upload_attachment_to_incident(self, incident_id: str, file_handle):
 		"""
 		Upload een attachment naar een incident in topdesk.
-		:param filename: string incl. extensie
-		:param incident_id: int
+		:param incident_id: str
 		:param file_handle: File. Bijvoorbeeld zoals in request.FILES
 		:return: response
 		"""
@@ -352,18 +366,21 @@ class TopdeskConnector:
 			status="secondLine",
 			costs: int = 0,
 			caller_name: str = "Enigma",
+			caller_id: str = "", # Deze override caller_name als deze is geset.
 			call_type: str = "Storing",
 			object_id: str = "",
 			object_name: str = "",
 			entry_type: str = "",
 			operator_name: str = "",
 			operator_group: str = "",
+			operator_group_id: str = "", # Deze override operator_group hierboven als deze geset is.
 			supplier_id: str = "",
 			category_name: str = "",
 			subcategory_name: str = "",
-			impact: str = "",
-			urgency: str = "",
-			priority: str = ""
+			impact_name: str = "",
+			urgency_name: str = "",
+			priority_name: str = "",
+			processing_status_name: str = "",
 	):
 		""" Gebruik object_name OF object_id, niet beide. """
 		if object_name and object_id:
@@ -376,6 +393,10 @@ class TopdeskConnector:
 			"caller": {"dynamicName": f"{caller_name}"},
 			"briefDescription": f"{short_description}"
 		}
+		if caller_id: # Deze override caller_name als deze is geset.
+			body["caller"] = {
+				"id": f"{caller_id}"
+			}
 		if call_type:
 			body["callType"] = {
 				"name": f"{call_type}"
@@ -384,9 +405,9 @@ class TopdeskConnector:
 			body["entryType"] = {
 				"name": f"{entry_type}"
 			}
-		if impact:
+		if impact_name:
 			body["impact"] = {
-				"id": f"{impact}"
+				"name": f"{impact_name}"
 			}
 		if object_id:
 			body["object"] = {
@@ -396,13 +417,13 @@ class TopdeskConnector:
 			body["object"] = {
 				"name": f"{object_name}"
 			}
-		if urgency:
+		if urgency_name:
 			body["urgency"] = {
-				"name": f"{urgency}"
+				"name": f"{urgency_name}"
 			}
-		if priority:
+		if priority_name:
 			body["priority"] = {
-				"name": f"{priority}"
+				"name": f"{priority_name}"
 			}
 		if costs:
 			body['costs'] = f"{costs}"
@@ -418,6 +439,10 @@ class TopdeskConnector:
 			body["operatorGroup"] = {
 				"name": f"{operator_group}"
 			}
+		if operator_group_id: # Deze override operator_group hierboven als deze geset is.
+			body["operatorGroup"] = {
+				"id": f"{operator_group_id}"
+			}
 		if operator_name:
 			body["operator"] = {
 				"name": f"{operator_name}"
@@ -426,6 +451,10 @@ class TopdeskConnector:
 			body['supplier'] = {
 				'id': supplier_id
 			}
+		if processing_status_name:
+			body['processingStatus'] = {
+				"name": f"{processing_status_name}"
+			}
 
 		return self.make_request('post', 'incidents/', json_data=body)
 
@@ -444,12 +473,29 @@ class TopdeskConnector:
 		operators = self.make_request('get', 'operators/', params={'page_size': 100})
 		return [(operator.get('id'), operator.get('dynamicName')) for operator in operators]
 
-	def suppliers(self):
+	def suppliers(self, page_size: int = 100, start: int = 0, exaustive=False):
 		"""
 		Get all supplier contacts
-		:return: dict
+
+		Args:
+			page_size (int): Hoeveelheid suppliers om op te vragen in request (max = 100)
+			start (int): start offset.
+			exaustive (bool): Als deze True is, gebruik sliding window om alle suppliers op te halen als er meer dan 100+ zijn.
+
+		:return: dict suppliers.
 		"""
-		return self.make_request('GET', 'suppliers/')
+		if not exaustive:
+			return self.make_request('GET', 'suppliers/', params={'page_size': page_size, 'start': start})
+		else:
+			output = []
+			response = self.make_request('GET', 'suppliers/', auto_parse=False, params={'page_size': page_size, 'start': start})
+			output.extend(json.loads(response.content))
+			while response.status_code == 206:
+				start = start + page_size -1
+				response = self.make_request('GET', 'suppliers/', auto_parse=False, params={'page_size': page_size, 'start': start})
+				output.extend(json.loads(response.content))
+			return output
+
 
 	def supplier(self, id):
 		"""
@@ -484,9 +530,9 @@ class TopdeskConnector:
 		params = {
 			'pageSize': ammount
 		}
-		return self.make_request('get', 'operationalActivities', params=params).get('result')
+		return self.make_request('get', 'operationalActivities', params=params).get('result', None)
 
-	def operational_activities_by_type(self, activity_type: str = None, ammount: int = 5) -> dict:
+	def operational_activities_by_type(self, activity_type: str = None, amount: int = 5, start_offset: int = 0) -> list:
 		"""
 		Verkrijg een lijst van alle operationele activiteiten op basis van soort
 		Params
@@ -495,15 +541,45 @@ class TopdeskConnector:
 		Returns: Dict
 
 		"""
-		params = f"query=type.name=='{activity_type}'&pageSize={ammount}&fields=all"
+		params = f"query=type.name=='{activity_type}'&pageSize={amount}&pageStart={start_offset}&fields=all"
+		print(params)
 		headers = self.headers.copy()
 		headers['Accept'] = "application/x.topdesk-om-activity-v1+json"
-		return self.make_request('get', 'operationalActivities', params=params, headers=headers)
+		result = self.make_request('get', 'operationalActivities', params=params, headers=headers)
+		output = []
+		if result is not None:
+			for activity in result.get('results', []):
+				output.append(self.make_request('get', f'operationalActivities/{activity.get("id")}'))
+		return output
+
+	def operational_activity_by_supplier(self, supplier_id: str, status_string: str, resolved: bool or None = None):
+		"""
+		Verkrijg operationele activiteiten op basis van supplier_id en status string.
+		De API van topdesk is best guur in dit opzicht. de call vereist de Accept: application/x.topdesk-om-activity-v1+json header
+		maar wanneer je deze set, krijg je alleen de id en number field terug. Dus moeten we deze nogmaals handmatig opvragen
+		om over alle informatie te beschikken.
+		Args:
+			supplier_id: id van de supplier gekoppeld aan de operationele activiteit.
+			status_string: string value van de status waarop gefilterd wordt
+			resolved: filter op resolved of niet.
+		"""
+		params = f"query=supplier.id=='{supplier_id}';type.name=='{status_string}'"
+		if resolved is not None:
+			params += f";resolved.value=='{resolved}'"
+		headers = self.headers.copy()
+		headers['Accept'] = "application/x.topdesk-om-activity-v1+json"
+		result = self.make_request('get', 'operationalActivities', params=params, headers=headers)
+		output = []
+		if result:
+			for activity in result.get('results', []):
+				output.append(self.make_request('get', f'operationalActivities/{activity.get("id")}'))
+		return output
 
 	def operational_activity_with_status(self, activity_id: str) -> dict:
 		"""
-		Verkrijg informatie over 1 operational activity. Deze is specifiek gemaakt omdat er geen headers meegestuurd
-		moeten worden om de status te verkrijgen...
+		Verkrijg informatie over 1 operational activity. Er worden twee requests gestuurd, een zonder x.topdesk header en eentje
+		met. Door een onhandigheid in de api van topdesk krijg je verschillende informatie terug afhankelijk van de header.
+		Dus versturen we twee keer een request en joinen we die samen.
 
 		Args:
 			activity_id:
@@ -511,10 +587,13 @@ class TopdeskConnector:
 		Returns: dict
 
 		"""
-		headers = self.headers.copy()
-		headers['Accept'] = "application/x.topdesk-om-activity-v1+json"
+		other_headers = self.headers.copy()
+		other_headers['Accept'] = "application/x.topdesk-om-activity-v1+json"
 		params = {"fields": "all"}
-		return self.make_request('get', f'operationalActivities/{activity_id}/', headers=headers, params=params)
+		request1 = self.make_request('get', f'operationalActivities/{activity_id}/', headers=other_headers, params=params)
+		request2 = self.make_request('get', f'operationalActivities/{activity_id}/', headers=self.headers, params=params)
+		request1.update(request2)
+		return request1
 
 	def operational_activity(self, activity_id):
 		"""
@@ -579,28 +658,6 @@ class TopdeskConnector:
 
 		return self.make_request('POST', f'operationalActivities/{activity_id}', json_data=body)
 
-	def operational_activity_by_supplier(self, supplier_id: str, status_string: str, resolved: bool or None = None):
-		"""
-		Verkrijg operationele activiteiten op basis van supplier_id en status string.
-		De API van topdesk is best guur in dit opzicht. de call vereist de Accept: application/x.topdesk-om-activity-v1+json header
-		maar wanneer je deze set, krijg je alleen de id en number field terug. Dus moeten we deze nogmaals handmatig opvragen
-		om over alle informatie te beschikken.
-		Args:
-			supplier_id: id van de supplier gekoppeld aan de operationele activiteit.
-			status_string: string value van de status waarop gefilterd wordt
-			resolved: filter op resolved of niet.
-		"""
-		params = f"query=supplier.id=={supplier_id};type.name=={status_string}"
-		if resolved is not None:
-			params += f";resolved.value=={resolved}"
-		headers = self.headers.copy()
-		headers['Accept'] = "application/x.topdesk-om-activity-v1+json"
-		result = self.make_request('get', 'operationalActivities', params=params, headers=headers)
-		output = []
-		for activity in result['results']:
-			output.append(self.make_request('get', f'operationalActivities/{activity.get("id")}'))
-		return output
-
 	def operational_activity_upload_file(self, activity_id, file):
 		"""
 		UPload een file naar een operationele activiteit.
@@ -699,3 +756,24 @@ class TopdeskConnector:
 		"""
 		assets = self.make_request('GET', 'assetmgmt/assets/')
 		return [(asset.get('text'), asset.get('text')) for asset in assets.get('dataSet')]
+
+	def asset_get_assigned_location(self, asset_id: str):
+		"""
+		Return de gekoppelde ruimte locatie aan een asset. Dit is een widget en moet via de assignments
+		route opgevraagd worden.
+		Args:
+			asset_id: Asset id string waarvan je de locatie wilt weten.
+
+		Returns: #todo
+
+		"""
+		assignments = self.make_request('GET', f'assetmgmt/assets/{asset_id}/assignments')
+		output = []
+		for item in assignments.get('locations', []):
+			x = {}
+			if item.get('location', None):
+				x.update({'location': item.get('location').get('name')})
+			if item.get('branch', None):
+				x.update({'branch': item.get('branch').get('name')})
+			output.append(x)
+		return output
